﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Munq.LifetimeManagers;

namespace Common
{
    public class MunqServiceFactory :IServiceFactory
    {
        public MunqServiceFactory(Munq.IocContainer container)
        {
            _container = container;
        }

        public IServiceFactory RegisterType<TTo, TFrom>() where TFrom : TTo, new()
        {
            _container.Register(typeof (TTo), typeof (TFrom)).WithLifetimeManager(new ContainerLifetime());
            return this;
        }

        public IServiceFactory RegisterType<TTo, TFrom>(string name) where TFrom : TTo, new()
        {
            _container.Register(name,typeof(TTo), typeof(TFrom)).WithLifetimeManager(new AlwaysNewLifetime());
            return this;
        }

        public IServiceFactory RegisterSingleton<TTo, TFrom>()
        {
            _container.Register(typeof (TTo), typeof (TFrom)).WithLifetimeManager(new ContainerLifetime());
            return this;
        }

        public IServiceFactory RegisterType<TTo>(Func<TTo> func )
        {
            _container.Register(typeof (TTo),(r)=> func()).WithLifetimeManager(new ContainerLifetime());
            return this;
        }

        private readonly Munq.IocContainer _container;
    }
}
