﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using mosquitto.net;

namespace Common.Views
{
    public class BrokerSelectorViewModel :NotificationObject
    {
        private bool _connecting;
        private string _version;
        private string _connectText;
        private string _address;
        private string _port;
        private string _username;
        private string _password;
        private string _error;
        private readonly IMosquitto _mqtt;
        private readonly IEventAggregator _eventAggregator;

        public BrokerSelectorViewModel()
        {
            ConnectCommand = new DelegateCommand(Connect, CanConnect);
            ConnectText = "Connect";
            Address = "test.mosquitto.org";
            Port = "1883";
            Version = "0.15.9";
            Error = "Error";
        }

        public BrokerSelectorViewModel(IMosquitto mqtt,IEventAggregator eventAggregator):this()
        {
            Version = "";
            Error = "";
            _mqtt = mqtt;
            _mqtt.OnConnection += new Action<IMosquitto, ConnectionResult>(_mqtt_OnConnection);
            _mqtt.OnDisconnect += new Action<IMosquitto, ReturnCode>(_mqtt_OnDisconnect);
            _mqtt.OnError += new Action<IMosquitto, ReturnCode>(_mqtt_OnError);
            _eventAggregator = eventAggregator;
            
        }

        void _mqtt_OnError(IMosquitto arg1, ReturnCode arg2)
        {
            Connecting = false;
            ConnectText = "Connect";
            ConnectCommand = new DelegateCommand(Connect, CanConnect);
            RaisePropertyChanged(() => ConnectCommand);
            
        }

        void _mqtt_OnDisconnect(IMosquitto arg1, ReturnCode arg2)
        {
            Connecting = false;
            ConnectText = "Connect";
            ConnectCommand = new DelegateCommand(Connect, CanConnect);
            RaisePropertyChanged(() => ConnectCommand);
        }

        void _mqtt_OnConnection(IMosquitto arg1, ConnectionResult arg2)
        {
            Connecting = false;
            ConnectText = "Disconnect";
            ConnectCommand = new DelegateCommand(Disconnect,()=>true);
            RaisePropertyChanged(() => ConnectCommand);
        }

        public string Address
        {
            get { return _address; }
            set
            {
                _address = value;
                RaisePropertyChanged(()=> Address);
                ConnectCommand.RaiseCanExecuteChanged();
            }
        }

        public string Port
        {
            get { return _port; }
            set
            {
                _port = value;
                RaisePropertyChanged(()=>Port);
                ConnectCommand.RaiseCanExecuteChanged();
            }
        }

        public DelegateCommand ConnectCommand { get; set; }

        public string ConnectText
        {
            get { return _connectText; }
            set
            {
                _connectText = value;
                RaisePropertyChanged(() => ConnectText);
            }
        }

        public string Error
        {
            get { return _error; }
            set
            {
                _error = value;
                RaisePropertyChanged(()=>Error);
            }
        }

        public string Version
        {
            get { return _version; }
            set
            {
                _version = string.Format("V:{0}",value);
                RaisePropertyChanged(()=>Version);
            }
        }

        public bool Connecting
        {
            get { return _connecting; }
            set
            {
                _connecting = value;
                RaisePropertyChanged(()=>Connecting);
            }
        }

        public string Username
        {
            get { return _username; }
            set
            {
                _username = value;
                RaisePropertyChanged(()=>Username);
            }
        }

        public string Password
        {
            get { return _password; }
            set
            {
                _password = value;
                RaisePropertyChanged(()=>Password);
            }
        }


        private void Connect()
        {
            Error = "";
            Connecting = true;
            _mqtt.SetUserNamePassword(Username, Password);
            _mqtt.ConnectAsync(Address, int.Parse(Port), 60,OnConnect);
            //if (rc)
            //{
            //    _mqtt.Start();
            //    _eventAggregator.GetEvent<BrokerConnectedEvent>().Publish(true);
            //    Version = _mqtt.Version;
            //}
            //else
            //{
            //    Error = rc.ToString();
            //    Connecting = false;
            //}
        }

        private void OnConnect(ReturnCode rc)
        {
            Connecting = false;
            if (rc)
            {
                _mqtt.Start();
                _eventAggregator.GetEvent<BrokerConnectedEvent>().Publish(true);
                Version = _mqtt.Version;
            }
            else
            {
                Error = rc.ToString();
                
            }
        }

        private void Disconnect()
        {
            if (_mqtt.Disconnect())
            {
                _eventAggregator.GetEvent<BrokerConnectedEvent>().Publish(false);
            }
        }

        private bool CanConnect()
        {
            bool connectOk = !string.IsNullOrEmpty(Address) && !string.IsNullOrEmpty(Port);
            return connectOk;
        }
    }
}
