﻿using System;
using System.Collections.Generic;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using mosquitto.net;
using System.Reactive.Linq;
namespace Common.Views
{
    public class BrokerStatusViewModel : NotificationObject
    {
        private string _version;
        private string _uptime;
        private readonly IMosquitto _mqtt;
        private readonly IEventAggregator _eventAggregator;
        private readonly List<IDisposable> _subscriptions = new List<IDisposable>();
        public string Version
        {
            get { return _version; }
            set
            {
                _version = value;
                RaisePropertyChanged(()=>Version);
            }
        }

        public string Uptime
        {
            get { return _uptime; }
            set
            {
                _uptime = value;
                RaisePropertyChanged(()=>Uptime);
            }
        }

        public BrokerStatusViewModel()
        {
            Version = "Unknown";
            Uptime = "0 hours";
        }

        public BrokerStatusViewModel(IMosquitto mqtt,IEventAggregator eventAggregator)
        {
            _mqtt = mqtt;
            _eventAggregator = eventAggregator;
            _eventAggregator.GetEvent<BrokerConnectedEvent>().Subscribe(OnConnect);
        }

        private void OnConnect(bool connected)
        {
            if(connected)
            {
                IObservable<Message> observable = _mqtt.Subscribe("$SYS/broker/+", Qos.Once);
                var sub = observable.Where((m) => m.Topic == "$SYS/broker/version").Subscribe((m) => Version = Message.ToAscii(m.Payload));
                _subscriptions.Add(sub);
                sub = observable.Where((m) => m.Topic == "$SYS/broker/uptime").Subscribe((m) => Uptime = Message.ToAscii(m.Payload));
                _subscriptions.Add(sub);
                _subscriptions.Add(observable.Subscribe((m) => _eventAggregator.GetEvent<MqttMessageReceived>().Publish(m)));
            }
            else
            {
                foreach(var s in _subscriptions)
                {
                    s.Dispose();
                }
                _subscriptions.Clear();
                Version = "";
                Uptime = "";

            }
        }
    }
}
