﻿using Common.Views;
using Microsoft.Practices.Prism.Modularity;
using Microsoft.Practices.Prism.Regions;
using Microsoft.Practices.ServiceLocation;
using Microsoft.Practices.Unity;

using Common;
using Module1.Views;

namespace Module1
{
    public class ModuleInit : IModule
    {
        private readonly IServiceLocator _locator;
        private readonly IServiceFactory _factory;
        private readonly IRegionManager _regionManager;

        public ModuleInit(IServiceLocator locator, IServiceFactory factory)
        {
            _factory = factory;
            _locator = locator;
            _regionManager = ServiceLocator.Current.GetInstance<IRegionManager>();
        }

        #region IModule Members

        public void Initialize()
        {
            int pid = System.Diagnostics.Process.GetCurrentProcess().Id;
            _factory.RegisterType(() => mosquitto.net.MosquittoFactory.Default(pid.ToString(), false));
            _regionManager.RegisterViewWithRegion(RegionNames.Top, () => _locator.GetInstance<BrokerSelector>());
            _regionManager.RegisterViewWithRegion(RegionNames.Status, () => _locator.GetInstance<BrokerStatusView>());
            _regionManager.RegisterViewWithRegion(RegionNames.Dashboard, () => _locator.GetInstance<DashboardView>());
            _regionManager.RegisterViewWithRegion(RegionNames.Subscriptions, () => _locator.GetInstance<SubscriptionsView>());
            _regionManager.RegisterViewWithRegion(RegionNames.Publish, () => _locator.GetInstance<PublishView>());
            
        }

        #endregion
    }
}
