﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reactive.Linq;
using System.Text;
using System.Windows;
using System.Windows.Threading;
using Common;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using mosquitto.net;

namespace Module1.ViewModels
{
    class DashboardViewModel :NotificationObject
    {
        public DashboardViewModel()
        {
            BytesReceivedPerSecond = 25;
            MessagesSent.Add(new KeyValuePair<DateTime, double>(DateTime.Now,2000000));
            MessagesReceived.Add(new KeyValuePair<DateTime, double>(DateTime.Now, 3000000));
            ActiveClients = 600;
            TotalClients = 0.7;
        }

        public DashboardViewModel(IEventAggregator aggregator, IMosquitto mqtt)
        {
            _aggregator = aggregator;
            _mqtt = mqtt;
            _aggregator.GetEvent<BrokerConnectedEvent>().Subscribe(OnConnect);
            _perSecondMeters = new List<double>(4) { 0, 0, 0, 0, };
        }

        

        public double BytesReceivedPerSecond
        {
            get { return _perSecondMeters[0]; }
            set
            {
                _perSecondMeters[0] = value;
                RaisePropertyChanged(() => BytesReceivedPerSecond);
                
            }
        }

        public double BytesSentPerSecond
        {
            get { return _perSecondMeters[1]; }
            set
            {
                _perSecondMeters[1] = value;
                RaisePropertyChanged(() => BytesSentPerSecond);
            }
        }

        public double MsgReceivedPerSecond
        {
            get { return _perSecondMeters[2]; }
            set 
            {
                _perSecondMeters[2] = value;
                RaisePropertyChanged(() => MsgReceivedPerSecond);
            }
        }

        public double MsgSentPerSecond
        {
            get { return _perSecondMeters[3]; }
            set
            {
                _perSecondMeters[3] = value;
                RaisePropertyChanged(() => MsgSentPerSecond);
            }
        }

        public double MsgInFlight
        {
            get { return _msgInFlight; }
            set
            {
                _msgInFlight = value;
                RaisePropertyChanged(() => MsgInFlight);
            }
        }

        public ObservableCollection<KeyValuePair<DateTime, double>> MessagesSent
        {
            get { return _messagesSent; }
            set { _messagesSent = value; }
        }

        public ObservableCollection<KeyValuePair<DateTime, double>> MessagesReceived
        {
            get { return _messagesReceived; }
            set { _messagesReceived = value; }
        }

        public int ActiveClients
        {
            get { return _activeClients; }
            set
            {
                _activeClients = value;
                RaisePropertyChanged(() => ActiveClients);
            }
        }

        public double TotalClients
        {
            get { return _totalClients; }
            set
            {
                _totalClients = value;
                RaisePropertyChanged(() => TotalClients);
            }
        }

        private void OnConnect(bool connected)
        {
            if (connected)
            {
                var observable = _mqtt.Subscribe("$SYS/broker/bytes/+/+", Qos.Once);
                _subscriptions.AddRange(new[]
                                            {
                                                observable.Where((m) => m.Topic == "$SYS/broker/bytes/per second/received").Subscribe((m) =>BytesReceivedPerSecond = double.Parse(Message.ToAscii(m.Payload))),
                                                observable.Where((m) => m.Topic == "$SYS/broker/bytes/per second/sent").Subscribe((m) => BytesSentPerSecond = double.Parse(Message.ToAscii(m.Payload)))
                                            });
               
                _subscriptions.Add(observable.Subscribe((m) => _aggregator.GetEvent<MqttMessageReceived>().Publish(m)));

                observable = _mqtt.Subscribe("$SYS/broker/messages/+/+", Qos.Once);
                _subscriptions.AddRange(new[]
                                            {
                                                observable.Where((m) => m.Topic == "$SYS/broker/messages/per second/received").Subscribe((m) =>MsgReceivedPerSecond = double.Parse(Message.ToAscii(m.Payload))),
                                                observable.Where((m) => m.Topic == "$SYS/broker/messages/per second/sent").Subscribe((m) => MsgSentPerSecond = double.Parse(Message.ToAscii(m.Payload))),
                                                
                                            });

                observable = _mqtt.Subscribe("$SYS/broker/messages/+", Qos.Once);
                _subscriptions.Add(observable.Where((m) => m.Topic == "$SYS/broker/messages/inflight").Subscribe((m) => MsgInFlight = double.Parse(Message.ToAscii(m.Payload))));
                _subscriptions.Add(observable.Where((m) => m.Topic == "$SYS/broker/messages/sent").Subscribe((m) => AddMessages("MessagesSent", MessagesSent, double.Parse(Message.ToAscii(m.Payload)))));
                _subscriptions.Add(observable.Where((m) => m.Topic == "$SYS/broker/messages/received").Subscribe((m) => AddMessages("MessagesReceived", MessagesReceived, double.Parse(Message.ToAscii(m.Payload)))));
                _subscriptions.Add(observable.Subscribe((m) => _aggregator.GetEvent<MqttMessageReceived>().Publish(m)));
                observable = _mqtt.Subscribe("$SYS/broker/clients/+", Qos.Once);
                _subscriptions.Add(observable.Where((m) => m.Topic == "$SYS/broker/clients/active").Subscribe((m) => ActiveClients = int.Parse(Message.ToAscii(m.Payload))));
                _subscriptions.Add(observable.Where((m) => m.Topic == "$SYS/broker/clients/total").Subscribe((m) => TotalClients = double.Parse(Message.ToAscii(m.Payload))/1500));
                _subscriptions.Add(observable.Subscribe((m) => _aggregator.GetEvent<MqttMessageReceived>().Publish(m)));
            }
            else
            {
                foreach (var s in _subscriptions)
                {
                    s.Dispose();
                }
                _perSecondMeters = new List<double>(4) { 0, 0, 0, 0, };
                RaisePropertyChanged("");
               
            }
        }

        private void AddMessages(string property, ObservableCollection<KeyValuePair<DateTime, double>> collection, double value)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(delegate()
                                                                      {
                                                                          if (collection.Count > 0)
                                                                          {
                                                                              var span =
                                                                                  (collection.Last().Key -
                                                                                   collection.First().Key).TotalHours;

                                                                              if (span > 12)
                                                                                  collection.RemoveAt(0);
                                                                          }
                                                                          collection.Add(new KeyValuePair<DateTime, double>(DateTime.Now, value));
                                                                          RaisePropertyChanged(property);
                                                                      }));
        
            
        }


        private List<double> _perSecondMeters = new List<double>(4){0,0,0,0,};
        private double _msgInFlight;
        private int _activeClients;
        private double _totalClients;
        private ObservableCollection<KeyValuePair<DateTime, double>> _messagesSent = new ObservableCollection<KeyValuePair<DateTime, double>>();
        private ObservableCollection<KeyValuePair<DateTime, double>> _messagesReceived = new ObservableCollection<KeyValuePair<DateTime, double>>();
        private readonly IEventAggregator _aggregator;
        private readonly IMosquitto _mqtt;
        private readonly List<IDisposable> _subscriptions = new List<IDisposable>();
    }
}
