﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using mosquitto.net;

namespace Module1.ViewModels
{
    class PublishViewModel :NotificationObject
    {

        private string _topic;
        private string _content;
        private Qos _qos;
        private bool _retain;
        private bool _canPublish;
        private readonly IEventAggregator _aggregator;
        private readonly IMosquitto _mqtt;

        public PublishViewModel()
        {
            PublishCommand = new DelegateCommand(Publish, () => CanPublish);
            Qos = Qos.Once;
            CanPublish = false;
        }

        public PublishViewModel(IEventAggregator aggregator, IMosquitto mqtt):this()
        {
            _aggregator = aggregator;
            _mqtt = mqtt;
            _aggregator.GetEvent<BrokerConnectedEvent>().Subscribe(OnBrokerConnected,ThreadOption.UIThread,true);
            
        }

        public DelegateCommand PublishCommand { get; set; }

        public string Topic
        {
            get { return _topic; }
            set
            {
                _topic = value;
                RaisePropertyChanged(()=>Topic);
            }
        }

        public string Content
        {
            get { return _content; }
            set
            {
                _content = value;
                RaisePropertyChanged(()=>Content);
            }
        }

        public Qos Qos
        {
            get { return _qos; }
            set
            {
                _qos = value;
                RaisePropertyChanged(()=>Qos);
            }
        }

        public bool Retain
        {
            get { return _retain; }
            set
            {
                _retain = value;
                RaisePropertyChanged(()=>Retain);
            }
        }

        public bool CanPublish
        {
            get { return _canPublish; }
            set
            {
                _canPublish = value;
                RaisePropertyChanged(() => CanPublish);
            }
        }

        private void Publish()
        {
            ushort mid = 0;
            _mqtt.Publish(ref mid, Topic, Encoding.UTF8.GetBytes(Content), Qos, Retain);
        }

        private void OnBrokerConnected(bool connected)
        {
            CanPublish = connected;
            PublishCommand.RaiseCanExecuteChanged();
        }
    }
}
