﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Windows;
using Common;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.ViewModel;
using mosquitto.net;

namespace Module1.ViewModels
{
    public class SubscriptionsViewModel :NotificationObject
    {

        public SubscriptionsViewModel()
        {
            SubscribeCommand = new DelegateCommand(Subscribe,()=>CanSubscribe);
            UnsubscribeCommand = new DelegateCommand<SubscriptionViewModel>(Unsubscribe, (vm) => CanSubscribe);
            Subscriptions.CollectionChanged += SubscriptionsCollectionChanged;
            Subscriptions.Add(new SubscriptionViewModel("Test Topic"));
        }

        public SubscriptionsViewModel(IEventAggregator aggregator, IMosquitto mqtt):this()
        {
            _aggregator = aggregator;
            _mqtt = mqtt;
            _aggregator.GetEvent<BrokerConnectedEvent>().Subscribe(OnConnected);
            //_mqtt.OnMessage += _mqtt_OnMessage;
        }

        public string Topic
        {
            get { return _topic; }
            set
            {
                _topic = value;
                RaisePropertyChanged(()=>Topic);
            }
        }

        public Qos Qos
        {
            get { return _qos; }
            set
            {
                _qos = value;
                RaisePropertyChanged(()=>Qos);
            }
        }

        public ObservableCollection<SubscriptionViewModel> Subscriptions
        {
            get { return _subscriptions; }
            set { _subscriptions = value; }
        }

        public DelegateCommand SubscribeCommand { get; private set; }

        public DelegateCommand<SubscriptionViewModel> UnsubscribeCommand { get; private set; }
        
        private void Subscribe()
        {
            int mid = 0;
            if(_mqtt.Subscribe(ref mid,Topic, Qos))
            {
                Subscriptions.Add(new SubscriptionViewModel(Topic));
            }

        }

        private void Unsubscribe(SubscriptionViewModel vm)
        {
            if (vm != null)
            {
                Subscriptions.Remove(vm);
            }
        }

        private void _mqtt_OnMessage(IMosquitto arg1, Message arg2)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(()=> OnSubscription(arg2)));
            
        }

        private void OnSubscription(Message message)
        {
            
            var existing = (from SubscriptionViewModel sub in Subscriptions
                            where sub.Topic == message.Topic
                            select sub).FirstOrDefault();
            if(existing == null)
            {
                existing = new SubscriptionViewModel(message.Topic,false);
                Subscriptions.Add(existing);
            }
            existing.Messages.Add(new MessageViewModel(message));
            
        }

        private void OnConnected(bool connected)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() => Subscriptions.Clear()));
            CanSubscribe = connected;
            SubscribeCommand.RaiseCanExecuteChanged();
            UnsubscribeCommand.RaiseCanExecuteChanged();
            if(connected)
                _mqtt.OnMessage += _mqtt_OnMessage;
            else
                _mqtt.OnMessage -= _mqtt_OnMessage;
            
        }

        void SubscriptionsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if(e.Action == NotifyCollectionChangedAction.Remove)
            {
                foreach(SubscriptionViewModel vm in e.OldItems)
                {
                    int mid=0;
                    _mqtt.Unsubscribe(ref mid, vm.Topic);
                }
            }
            RaisePropertyChanged(() => Subscriptions);
            
        }

        private bool CanSubscribe { get; set; }
        private string _topic;
        private Qos _qos;
        private ObservableCollection<SubscriptionViewModel> _subscriptions = new ObservableCollection<SubscriptionViewModel>();
        private readonly IEventAggregator _aggregator;
        private readonly IMosquitto _mqtt;
        private bool _disposed;
        
    }
}
