﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reactive.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace mosquitto.net.subscribe_client
{
    class Program
    {
        static private Arguments _arguments;
        static void Main(string[] args)
        {
            if (args.Length != 1)
                return;
            _arguments = new Arguments(new Uri(args[0].Contains("mqtt") ? args[0] : "mqtt:" + args[0]), System.Diagnostics.Process.GetCurrentProcess().Id);
            System.Console.WriteLine("Qos={0}", _arguments.Qos);
            System.Console.WriteLine("debug={0}",_arguments.Debug);
            System.Console.WriteLine("username={0}", _arguments.Username);
            System.Console.WriteLine("topic={0}", _arguments.Topic);
            var mqtt = MosquittoFactory.Default(_arguments.Id, _arguments.Clean);
            {
                
                System.Console.WriteLine(mqtt.Version);
                if(_arguments.Debug)
                   mqtt.InitialiseLog();
                if (!string.IsNullOrEmpty(_arguments.Username))
                    mqtt.SetUserNamePassword(_arguments.Username, _arguments.Password);
                mqtt.OnError += new Action<IMosquitto, ReturnCode>(mqtt_OnError);
                mqtt.OnConnection += mqtt_OnConnection;
                mqtt.OnDisconnect += mqtt_OnDisconnect;
                mqtt.OnLog += new Action<IMosquitto, LogPriority, string>(mqtt_OnLog);
                ReturnCode rc = mqtt.Connect(_arguments.Host, _arguments.Port, _arguments.KeepAlive);
                if (rc == ReturnCode.Code.Success)
                {
                    mqtt.Start();
                    mqtt.OnSubscribe += mqtt_OnSubscribe;
                    mqtt.OnMessage += mqtt_OnMessage;
                    //using (var subscription = mqtt.Subscribe("$SYS/broker/bytes/+",
                    //                                        Qos.Handshake,
                    //                                            (m) =>
                    //                                            {
                    //                                                System.Console.WriteLine("Topic {0}", m.Topic);
                    //                                                System.Console.WriteLine("Value {0}", Encoding.ASCII.GetString(m.Payload));

                    //                                            },
                    //                                             (m) => m.Topic == "$SYS/broker/bytes/sent"))
                    //{



                    Subscribe(mqtt, _arguments);
                    System.Console.ReadKey();
                    
                }   
            }
        }

        static void mqtt_OnLog(IMosquitto arg1, LogPriority arg2, string arg3)
        {
            System.Console.WriteLine("{0}/{1}",arg2,arg3);
        }

        static void mqtt_OnError(IMosquitto mqtt, ReturnCode arg2)
        {
            System.Console.WriteLine("Error {0}", arg2);
            if (mqtt.Reconnect() == ReturnCode.Code.Success)
            {
                System.Console.WriteLine("Resubscribing");
                Subscribe(mqtt, _arguments);
            }
            else
                System.Console.WriteLine("Reconnect failed");
        }

        static void Subscribe(IMosquitto mqtt,Arguments args)
        {
            int id = 0;
            mqtt.Subscribe(ref id, args.Topic, args.Qos);
            
            
            //mqtt.Subscribe("$SYS/broker/bytes/sent", 0, mqtt_OnMessage);
            //mqtt.Subscribe("$SYS/broker/bytes/sent", 0, mqtt_OnMessage);
            //mqtt.Subscribe(ref id, uri.LocalPath.Substring(1), 0);
            //mqtt.Subscribe(ref id, "$SYS/broker/bytes/sent", 0);
            //mqtt.Subscribe(ref id, "$SYS/broker/bytes/sent", 0);

        }

        static void mqtt_OnDisconnect(IMosquitto arg1, ReturnCode arg2)
        {
            System.Console.WriteLine("Disconnected {0}", arg2);
            
        }

        

        static void mqtt_OnConnection(IMosquitto arg1, ConnectionResult arg2)
        {
            System.Console.WriteLine("Connected {0}",arg2.Description());
        }

        static void mqtt_OnMessage(IMosquitto arg1, Message arg2)
        {
            System.Console.WriteLine("# {0}", arg2.Mid);
            System.Console.WriteLine("Topic {0}", arg2.Topic);
            System.Console.WriteLine("Value {0}",Encoding.ASCII.GetString( arg2.Payload));
        }

        static void mqtt_OnSubscribe(IMosquitto arg1, int arg2, byte[] arg3)
        {
            System.Console.WriteLine("#Number subs. {0}", arg2);
        }

        //static void Main(string[] args)
        //{
        //    libmosquitto.mosquitto_lib_init();
        //    IntPtr ptr = new IntPtr(0x55AA);
        //    int maj;
        //    int min;
        //    int rev;
        //    libmosquitto.mosquitto_lib_version(out maj, out min, out rev);
        //    System.Console.WriteLine("{0}.{1}.{2}",maj,min,rev);
        //    _client = libmosquitto.mosquitto_new("", ptr);
        //    libmosquitto.mosquitto_log_init(_client, libmosquitto.MOSQ_LOG_ALL,libmosquitto.MOSQ_LOG_STDOUT);
        //    libmosquitto.mosquitto_connect_callback_set(_client,OnConnectCallback);
        //    libmosquitto.mosquitto_message_callback_set(_client,OnMessageCallback);
        //    int rc = libmosquitto.mosquitto_connect(_client, "test.mosquitto.org", 1883, 60, true);
        //    ushort id = 0;
            
        //    do
        //    {
        //        rc = libmosquitto.mosquitto_loop(_client, -1);
        //    } while (rc == 0);
        //    libmosquitto.mosquitto_destroy(_client);
        //    libmosquitto.mosquitto_lib_cleanup();
        //}

        //static void OnConnectCallback(IntPtr cookie, int rc)
        //{
        //    ushort id = 0;
        //    libmosquitto.mosquitto_subscribe(_client, ref id, "$SYS/broker/bytes/received", 0);
        //}

        //static void OnMessageCallback(IntPtr cookie, IntPtr message)
        //{
        //    _Message m = (_Message)Marshal.PtrToStructure(message, typeof (_Message));
        //    byte[] pl = new byte[m.payloadlen];
        //    Marshal.Copy(m.payload, pl, 0,(int) m.payloadlen);
        //    var text = Encoding.ASCII.GetString(pl);
        //    System.Console.WriteLine("$SYS/broker/bytes/received {0}",text);
        //}
    }
}
