﻿using System.Collections.Concurrent;
using System.Threading.Tasks;

namespace System.Threading
{
    /// <summary>
    /// Implementation og IActiveObject interface
    /// Uses a Blocking collection to queue tasks.
    /// Uses a Task to read queue and execute task synchronously
    /// </summary>
    public class ActiveObject : IActiveObject
    {
        #region IActiveObject Members

        public void Enqueue(Action action, Action callback, int delay = 0 )
        {
            var task = new Task(action);
            task.ContinueWith(t=>callback);
            EnqueueTask(new Task(action), delay);
            
        }

        public void Enqueue(Action action, int delay = 0)
        {
            Enqueue(new Task(action),delay);
        }

        public void Enqueue(Task task,int delay = 0)
        {
            EnqueueTask(task,delay);
        }

        public void Start()
        {
           TaskAction();

        }

        public void Stop()
        {
            Enqueue(default(Task));
            _task.Wait();

        }

        
        #endregion
        /// <summary>
        /// Constructor
        /// </summary>
        /// <example>
        /// var ao = new ActiveObject();
        /// ao.Start();
        /// ao.Enqueue(()=> System.Console.WriteLine("Hello World");
        /// ao.Stop();
        /// </example>
        public ActiveObject(string name)
        {
            _name = name;
        }

        public ActiveObject(string name,TaskScheduler schedular)
        {
            _name = name;
            _factory = new TaskFactory(schedular);
        }
        
        private void TaskAction()
        {
            Diagnostics.Debug.WriteLine(string.Format("{0} AO started", _name));
            Action<Task> recursive = null;
            recursive = (Task acedent) => {
                                    if (!_blockingCollection.IsCompleted)
                                    {
                                        //Diagnostics.Debug.WriteLine(string.Format("{0} on thread {1} with queue size of {2}",_name, Thread.CurrentThread.ManagedThreadId, _blockingCollection.Count));
                                        Task t = _blockingCollection.Take();

                                        if (t != null)
                                        {
                                            t.RunSynchronously();
                                            acedent.ContinueWith(recursive);
                                        }
                                        else
                                        {
                                            return;
                                        }

                                    }
                                   
                                };
            _task = _factory.StartNew(()=> { }).ContinueWith(recursive);
            
        }

        private void EnqueueTask(Task task, int delay)
        {
            if (delay > 0)
            {
                var timer = new Timers.Timer {AutoReset = false, Interval = delay};
                timer.Elapsed += (s, e) => _blockingCollection.TryAdd(task);
                timer.Enabled = true;
            }
            else
                _blockingCollection.TryAdd(task);
        }


        private Task _task;
        private readonly BlockingCollection<Task> _blockingCollection = new BlockingCollection<Task>();
        private readonly string _name = string.Empty;
        private TaskFactory _factory = new TaskFactory();
    }
}
