﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace mosquitto.net
{
    public class Arguments
    {
        public const int DefaultKeepAlive = 60;
        public const int DefaultPort = 1883;
        public const string DefaultHost = "localhost";
        public const string DefaultUsername = "";
        public const string DefaultPassword = null;
        public const bool DefaultDebug = false;
        public const bool DefaultClean = true;
        public const Qos  DefaultQos = Qos.Once;

        public Arguments(Uri uri, int processId)
        {
            _processId = processId;
            KeepAlive = 60;
            Username = null;
            Password = null;
            Qos = Qos.Once;
            Debug = false;
            Clean = true;
            Port = uri.Port;
            Host = uri.Host;
            Topic = uri.LocalPath.Substring(1);
            Id = string.Format("mosq_sub_{0}", _processId);
            string query = uri.Query;
            if(!string.IsNullOrEmpty(query))
            {
                query = query.Remove(0,1);
                var items = query.Split('&');
                foreach (var item in items)
                {
                    var kv = item.Split('=');
                    if(Handlers.ContainsKey(kv[0]))
                    {
                        Handlers[kv[0]](kv[0], kv[1]);
                    }
                }
            }
        }

        public int KeepAlive { get; private set; }

        public string Username { get; private set; }

        public string Password { get; private set; }

        public Qos Qos { get; private set; }

        public bool Debug { get; private set; }

        public bool Clean { get; private set; }

        public int Port { get; private set; }

        public string Topic { get; private set; }

        public string Host { get; private set; }

        private Dictionary<string, Action<string, string>> Handlers
        {
            get
            {
                return _handlers ?? (_handlers = new Dictionary<string, Action<string, string>>
                                                     {
                                                         {
                                                             "keepalive", (key, value) => { KeepAlive = int.Parse(value); }
                                                             },
                                                         {"username", (key, value) => { Username = value; }},
                                                         {"password", (key, value) => { Password = value; }},
                                                         {"qos", (key, value) => { Qos = (Qos) int.Parse(value); }},
                                                         {"debug", (key, value) => { Debug = bool.Parse(value); }},
                                                         {"clean", (key, value) => { Clean = bool.Parse(value); }},
                                                         {"id", (key, value) => { Id = value; }},
                                                         {
                                                             "prefix",
                                                             (key, value) => { Id = string.Format("{0}_{1}", value, _processId); }
                                                             }
                                                     });
            }
        }

        public string Id { get; private set; }

        private Dictionary<string,Action<string,string>> _handlers;

        private readonly int _processId;
    }
}
