﻿using System.ComponentModel;
using System.Reflection;

namespace mosquitto.net
{
    /// <summary>
    /// Result of connection to a broker
    /// </summary>
    public enum ConnectionResult
    {
        [Description("Success")] Success,
        [Description("Unacceptable protocol version")] IncorrectProtocol,
        [Description("Identifier rejected")] IndentifierRejected,
        [Description("Broker unavailable")] BrokerUnavailable,
        [Description("Bad username or password")] BadUsernameOrPassword,
        [Description("Not authorized")] NotAuthorized,
        [Description("Unknown")] Unknown,
    }

    public static class Extension
    {
        /// <summary>
        /// The description of the result
        /// </summary>
        /// <param name="value"></param>
        /// <returns>Text describing the result</returns>
        public static string Description(this ConnectionResult value)
        {
            FieldInfo fi = value.GetType().GetField(value.ToString());
            var attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute),false);
            if (attributes.Length > 0)
                return attributes[0].Description;
            return value.ToString();
        }
    }
}