﻿using System.Threading.Tasks;

namespace System.Threading
{
    /// <summary>
    /// Inteface for active object implementation
    /// </summary>
    public interface IActiveObject
    {
        /// <summary>
        /// Start Queue processing Task
        /// </summary>
        void Start();
        /// <summary>
        /// Stop the Queue processing Task
        /// </summary>
        void Stop();
        /// <summary>
        /// Enqueue an action after the specified delay
        /// </summary>
        /// <param name="action">Action to execute</param>
        /// <param name="delay">Time delay before queueing</param>
        void Enqueue(Action action, int delay = 0);
        /// <summary>
        /// Enqueue an action after the specified delay. Notify completion using callback
        /// </summary>
        /// <param name="action">Action to execute</param>
        /// <param name="callback">Action to execute on completion</param>
        /// <param name="delay">Time delay before queueing</param>
        void Enqueue(Action action, Action callback, int delay = 0 );
        /// <summary>
        /// Enqueue a task after the specified delay
        /// </summary>
        /// <param name="task">Task to execute</param>
        /// <param name="delay">Time delay before queueing</param>
        void Enqueue(Task task, int delay = 0);

    }
}
