#include "StdAfx.h"
#include "arguments.h"
#include <algorithm>
namespace mosq
{
	arguments::arguments(int pid):
	_keepalive(60),
	_host("localhost"),
	_topic("$SYS/broker/+/+/+"),
	_port(1883),
	_qos(once),
	_clean(true),
	_debug(false)
	{
		std::ostringstream strm;
		strm << "mosq_" << pid;
		_id = strm.str();
	};

	arguments::arguments(const uri& u, int pid):
	_keepalive(60),
	_host("localhost"),
	_topic("$SYS/broker/+/+/+"),
	_port(1883),
	_qos(once),
	_clean(true),
	_debug(false)
	{
		std::ostringstream strm;
		strm << "mosq_" << pid;
		_id = strm.str();
		create(u);
	}

	

	void arguments::create(const uri& u)
	{
		if(_handlers.size() == 0)
		{
			_handlers.insert( mapped_function("topic",[this] (const string& key,const string& value){ topic(value); }));
			_handlers.insert( mapped_function("username",[this] (const string& key,const string& value){ username(value); }));
			_handlers.insert( mapped_function("password",[this] (const string& key,const string& value){ password(value); }));
			_handlers.insert( mapped_function("qos",[this] (const string& key,const string& value){ quality(static_cast<qos>(atoi(value.c_str()))); }));
			_handlers.insert( mapped_function("keepalive",[this] (const string& key,const string& value){ keepalive(atoi(value.c_str())); }));
			_handlers.insert( mapped_function("debug",[this] (const string& key,const string& value){ debug(atoi(value.c_str())); }));
			_handlers.insert( mapped_function("clean",[this] (const string& key,const string& value){ debug(atoi(value.c_str())); }));
		}
		host(u.getHost());
		port(u.getPort());
		topic(u.getDecodedPath());
		uri_query query = u.getDecodedQueryComponents();
		std::for_each(query.begin(),query.end(),[this](pair<string,string> kvp){ this->find(kvp.first,kvp.second);});

	}

	void arguments::find(const string& key, const string& value)
	{
		if(_handlers.find(key) != _handlers.end())
		{
			_handlers[key](key,value);
		}
	}

	arguments::~arguments(void)
	{
	}
}
