#pragma once
#include <cstdint>
#include <list>
#include <vector>
#include <functional>
#include "connection_result.h"
#include "qos.h"
#include "return_code.h"
#include "log_destination.h"
#include "log_priority.h"
#include "message.h"

using namespace std;
namespace mosq
{
class mosquitto_base
{
public:

	std::list< std::function<void (const mosquitto_base&, connection_result)> > on_connect;
	std::list< std::function<void (const mosquitto_base&, return_code)> > on_disconnect;
	std::list< std::function<void (const mosquitto_base&, uint16_t mid)> > on_publish;
	std::list< std::function<void (const mosquitto_base&, uint16_t mid, vector<qos> granted_qos)> > on_subscribe;
	std::list< std::function<void (const mosquitto_base&, uint16_t mid)> > on_unsubscribe;
	std::list< std::function<void (const mosquitto_base&, const message& message)> > on_message;
	std::list< std::function<void ( const mosquitto_base&, int level, std::string& message)> > on_log;
	mosquitto_base(void);
	virtual ~mosquitto_base(void);

	virtual string version() = 0;
	virtual return_code initialise_log()=0;
	virtual return_code set_will( const string& topic, const vector<uint8_t>& payload, qos qos, bool retain)=0;
	virtual return_code set_username_password(const string& username, const string& password)=0;
	virtual return_code connect(const string& host, int port, int keepalive)=0;
	virtual void run()=0;
	virtual return_code reconnect()=0;
	virtual return_code disconnect()=0;
	virtual return_code subscribe(int* const mid,const string& sub, qos quality)=0;
	virtual return_code unsubscribe(int* const mid,const string& sub)=0;
	virtual return_code publish(int* const mid,const string& topic, const vector<uint8_t>& payload, qos quality, bool retain)=0;
};
}

