#include "StdAfx.h"
#include <algorithm>
#include <ppl.h>
#include "mosquitto_impl.h"
#include "mosquitto.h"
namespace mosq
{

	int mosquitto_impl::_initialized = mosquitto_impl::initialize();

	int mosquitto_impl::initialize()
	{
		return mosquitto_lib_init();
	}

	void mosquitto_impl::cleanup()
	{
		mosquitto_lib_cleanup();
	}

	mosquitto_impl::mosquitto_impl(const string& id,bool clean)
	{
		_mosq = mosquitto_new(id.c_str(),clean,this);
		mosquitto_connect_callback_set(static_cast<mosquitto*>(_mosq),mosquitto_impl::on_connect_callback);
		mosquitto_message_callback_set(static_cast<mosquitto*>(_mosq),mosquitto_impl::on_message_callback);
		atexit(mosquitto_impl::cleanup);
	}


	mosquitto_impl::~mosquitto_impl(void)
	{
		mosquitto_destroy(static_cast<mosquitto*>(_mosq));
		_task.cancel();
		
	}

	string mosquitto_impl::version()
	{
		int major = 0;
		int minor = 0;
		int release = 0;
		mosquitto_lib_version(&major,&minor,&release);
		std::ostringstream os;
		os << major << ":" << minor << ":" << release;
		return os.str();
	}

	return_code mosquitto_impl::initialise_log() 
	{
		mosquitto_log_callback_set(static_cast<mosquitto*>(_mosq),mosquitto_impl::on_log_callback);
		return successful;
	}

	return_code mosquitto_impl::set_will(const string& topic, const vector<uint8_t>& payload, qos qos, bool retain)
	{
		return mosquitto_will_set(static_cast<mosquitto*>(_mosq),topic.c_str(),payload.size(),&payload[0],qos,retain);
	}

	return_code mosquitto_impl::set_username_password(const string& username, const string& password)
	{
		return mosquitto_username_pw_set(static_cast<mosquitto*>(_mosq),username.c_str(),password.c_str());
	}

	return_code mosquitto_impl::connect(const string& host, int port, int keepalive)
	{
		return mosquitto_connect(static_cast<mosquitto*>(_mosq),host.c_str(),port,keepalive);
	}

	void mosquitto_impl::run()
	{
		_task.run([this](){run(this);});
	}

	void mosquitto_impl::run(mosquitto_impl* _this)
	{
		return_code rc = successful;
		do
		{
				rc = mosquitto_loop(static_cast<mosquitto*>(_this->_mosq),60,1);
			
		} while(rc == 0);
	}
	return_code mosquitto_impl::reconnect()
	{
		return mosquitto_reconnect(static_cast<mosquitto*>(_mosq));
	}

	return_code mosquitto_impl::disconnect()
	{
		return mosquitto_disconnect(static_cast<mosquitto*>(_mosq));
	}

	return_code mosquitto_impl::subscribe(int* const mid,const string& sub, qos quality)
	{
		return mosquitto_subscribe(static_cast<mosquitto*>(_mosq),mid,sub.c_str(),quality);
	}

	return_code mosquitto_impl::unsubscribe(int* const mid,const string& sub)
	{
		return mosquitto_unsubscribe(static_cast<mosquitto*>(_mosq),mid,sub.c_str());
	}

	return_code mosquitto_impl::publish(int* const mid,const string& topic, const vector<uint8_t>& payload, qos quality, bool retain)
	{
		return mosquitto_publish(static_cast<mosquitto*>(_mosq),mid,topic.c_str(),payload.size(),&payload[0],quality,retain);
	}

	void mosquitto_impl::on_log_callback(struct mosquitto *mosq, void *_this, int level, const char* message)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		for(auto callback = self->on_log.begin();callback != self->on_log.end();callback++)
		{
			(*callback)(*self,level,std::string(message));
		}
	}

	void mosquitto_impl::on_connect_callback(struct mosquitto *mosq, void *_this, int cr)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		for(auto callback = self->on_connect.begin();callback != self->on_connect.end();callback++)
		{
			(*callback)(*self,(connection_result)cr);
		}
	}
	
	void mosquitto_impl::on_disconnect_callback(struct mosquitto *, void *_this, int cr)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		for(auto callback = self->on_disconnect.begin();callback != self->on_disconnect.end();callback++)
		{
			(*callback)(*self,(connection_result)cr);
		}
	}
	
	void mosquitto_impl::on_publish_callback(struct mosquitto *, void *_this, uint16_t mid)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		for(auto callback = self->on_publish.begin();callback != self->on_publish.end();callback++)
		{
			(*callback)(*self,mid);
		}
	}
	
	void mosquitto_impl::on_subscribe_callback(struct mosquitto *, void *_this, uint16_t mid, int qos_count, const uint8_t *granted_qos)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		vector<qos> subscribers;
		for(int i = 0;i < qos_count;i++)
		{
			subscribers.push_back((qos)granted_qos[i]);
		}
		for(auto callback = self->on_subscribe.begin();callback != self->on_subscribe.end();callback++)
		{
			(*callback)(*self,mid,subscribers);
		}
	}

	void mosquitto_impl::on_unsubscribe_callback(struct mosquitto *, void *_this, uint16_t mid)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		for(auto callback = self->on_unsubscribe.begin();callback != self->on_unsubscribe.end();callback++)
		{
			(*callback)(*self,mid);
		}
	}

	void mosquitto_impl::on_message_callback(struct mosquitto *, void *_this, const struct mosquitto_message *msg)
	{
		mosquitto_impl* self = static_cast<mosquitto_impl*>(_this);
		mosq::message new_message(msg);
		Concurrency::parallel_for_each(self->on_message.begin(),self->on_message.end(),[&new_message,&self](std::function<void (const mosquitto_base&, const message& message)> callback )
		{
			(callback)(*self,new_message);
		});
	}

}
