#pragma once
#include <ppl.h>
#include "mosquitto_base.h"

namespace mosq
{
	class mosquitto_impl : public mosquitto_base
	{
	public:
		mosquitto_impl(const string& id,bool clean = true);
		~mosquitto_impl(void);

		string version();
		return_code initialise_log();
		return_code set_will(const string& topic, const vector<uint8_t>& payload, qos qos, bool retain) ;
		return_code set_username_password(const string& username, const string& password) ;
		return_code connect(const string& host, int port, int keepalive) ;
		void run() ;
		return_code reconnect() ;
		return_code disconnect() ;
		return_code subscribe(int* const mid,const string& sub, qos quality) ;
		return_code unsubscribe(int* const mid,const string& sub);
		return_code publish(int* const mid,const string& topic, const vector<uint8_t>& payload, qos quality, bool retain) ;

	private:
		Concurrency::task_group _task;
		void* _mosq;
		static int _initialized;
		static int initialize();
		static void cleanup();
		static void on_log_callback(struct mosquitto *mosq, void *_this, int level, const char* message);
		static void on_connect_callback(struct mosquitto *, void *, int);
		static void on_disconnect_callback(struct mosquitto *, void *, int);
		static void on_publish_callback(struct mosquitto *, void *, uint16_t);
		static void on_subscribe_callback(struct mosquitto *, void *, uint16_t, int, const uint8_t *);
		static void on_unsubscribe_callback(struct mosquitto *, void *, uint16_t);
		static void on_message_callback(struct mosquitto *, void *, const struct mosquitto_message *);
		static void run(mosquitto_impl* _this);
	};

}

