#include "StdAfx.h"
#include "uri.h"
#include "stringtokenizer.h"
//#include "stringutils.h"
#include <cctype>
#include <sstream>
#include <algorithm>
namespace mosq
{
const std::string uri::ALNUM("AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789");

std::string toLower(const std::string& str)
{
	std::string copy = str;
	std::transform(copy.begin(),copy.end(),copy.begin(),(int(*)(int)) tolower);
	return copy;
}

template<typename T>
std::string fromNumber(T number)
{
	std::ostringstream strm;
	strm << number;
	return strm.str();
}
uri::uri(const std::string& str)
:	m_is_good(!str.empty())
{
	std::string::size_type colon_pos = str.find(':');
	if (colon_pos == std::string::npos) {
		m_is_absolute = false;
		// m_scheme.erase();
		parseSchemeSpecificPart(str);
	} else {
		if (validateScheme(str.substr(0, colon_pos))) {
			m_is_absolute = true;
			m_scheme = toLower( str.substr(0, colon_pos) );
			//std::transform(m_scheme.data(),m_scheme.begin(),m_scheme.end(),m_scheme.begin(),::tolower);
			//m_scheme = StringUtils::toLowerStr(str.substr(0, colon_pos));
			parseSchemeSpecificPart(str.substr(colon_pos+1));
		} else {
			m_is_absolute = false;
			// m_scheme.erase();
			parseSchemeSpecificPart(str);
		}
		
	}
}

uri::uri(const uri& base, const uri& relative)
:	m_is_good(true)
{
	if (relative.isAbsolute()) {
		*this = relative;
		return;
	} else if (base.isAbsolute()) {
		m_is_absolute = true;
		m_scheme = base.m_scheme;
	} else {
		m_is_absolute = false;
		//m_scheme.erase();
	}
	if (!validateScheme(m_scheme)) {
		m_is_good = false;
	}
	if (relative.hasAuthority()) {
		m_has_authority = true;
		m_raw_authority = relative.m_raw_authority;
		m_has_user_info = relative.m_has_user_info;
		m_raw_user_info = relative.m_raw_user_info;
		m_host = relative.m_host;
		m_port = relative.m_port;
		m_is_relative_path = relative.m_is_relative_path;
		m_raw_path = relative.m_raw_path;
		m_has_query = relative.m_has_query;
		m_raw_query = relative.m_raw_query;
		m_has_fragment = relative.m_has_fragment;
		m_raw_fragment = relative.m_raw_fragment;
		m_raw_scheme_specific_part = relative.m_raw_scheme_specific_part;
		if (!validateHost(m_host)) {
			m_is_good = false;
		}
		if (!m_is_absolute && relative.m_raw_scheme_specific_part.empty()) {
			m_is_good = false;
		}
		return;
	} else if (base.hasAuthority()) {
		m_has_authority = true;
		m_raw_authority = base.m_raw_authority;
		m_has_user_info = base.m_has_user_info;
		m_raw_user_info = base.m_raw_user_info;
		m_host = base.m_host;
		m_port = base.m_port;
		if (!validateHost(m_host)) {
			m_is_good = false;
		}
	} else {
		m_has_authority = false;
		// m_raw_authority.erase();
		m_has_user_info = false;
		// m_raw_user_info.erase();
		// m_host.erase();
		m_port = -1;
	}
	if (!relative.isRelativePath() || base.isRelativePath()) {
		m_is_relative_path = relative.isRelativePath();
		m_raw_path = relative.m_raw_path;
		m_has_query = relative.m_has_query;
		m_raw_query = relative.m_raw_query;
		m_has_fragment = relative.m_has_fragment;
		m_raw_fragment = relative.m_raw_fragment;
		updateSchemeSpecificPart();
		if (!m_is_absolute && relative.m_raw_scheme_specific_part.empty()) {
			m_is_good = false;
		}
		return;
	} else {
		// base has absolute path, relative has relative path
		m_is_relative_path = false;
		std::string::size_type pos = base.m_raw_path.rfind('/');
		if (pos == std::string::npos) {
			//m_raw_path = "";
		} else {
			m_raw_path = base.m_raw_path.substr(0, pos+1);
			// pos+1 is for including the trailing slash
		}
		std::string rel_path = relative.m_raw_path;
		while (true) {
			if (rel_path.substr(0, 2) == "./") {
				rel_path.erase(0, 2);
			} else if (rel_path.substr(0, 3) == "../") {
				rel_path.erase(0, 3);
				std::string::size_type pos1 = m_raw_path.rfind('/', m_raw_path.length()-2);
				// length()-2 is needed to skip the trailing slash
				if (pos1 == std::string::npos) {
					m_raw_path.erase();
				} else {
					m_raw_path.erase(pos1+1);
				}
			} else {
				break;
			}
		}
		m_raw_path += rel_path;
	}
	m_has_query = relative.m_has_query;
	m_raw_query = relative.m_raw_query;
	m_has_fragment = relative.m_has_fragment;
	m_raw_fragment = relative.m_raw_fragment;
	updateSchemeSpecificPart();
	if (!m_is_absolute && relative.m_raw_scheme_specific_part.empty()) {
		m_is_good = false;
	}
}

std::string
uri::escape(const std::string& str, const char *allow_extra)
{
	static const std::string allow_chars("aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ0123456789-_.");
	static char const hexchars[] = "0123456789abcdef";
	const std::string& allow = allow_extra ? allow_chars+allow_extra : allow_chars;
	std::string res;
	res.reserve(str.size());
	std::string::size_type pos = 0, fpos;
	while (true) {
		fpos = str.find_first_not_of(allow, pos);
		if (fpos == std::string::npos)
			break;
		res.append(str, pos, fpos-pos);
		res += '%';
		unsigned char ch = str[fpos];
		res += hexchars[ch >> 4];
		res += hexchars[ch & 0xf];
		pos = fpos+1;
	}
	res.append(str, pos, str.length()-pos);
	return res;
}

std::string
uri::unescape(const std::string& str, bool decode_plus)
{
	static const std::string hexchars("0123456789aAbBcCdDeEfF");
	std::string res;
	std::string::size_type pos = 0, fpos;
	while (true) {
		fpos = decode_plus ? str.find_first_of("%+", pos) : str.find('%', pos);
		if (fpos == std::string::npos)
			break;
		res.append(str, pos, fpos-pos);
		if (str[fpos] == '%') {
			if (str.length() < fpos+3 ||
			    str.substr(fpos+1, 2).find_first_not_of(hexchars) != std::string::npos) {
				res += '%';
				pos = fpos+1;
				continue;
			}
			std::istringstream strm(str.substr(fpos+1, 2));
			unsigned int val = 0;
			strm >> std::hex >> val;
			res += (unsigned char)val;
			pos = fpos+3;
		} else { // str[fpos] == '+'
			res += ' ';
			pos = fpos+1;
		}
	}
	res.append(str, pos, str.length()-pos);
	return res;
}

void
uri::setScheme(const std::string& scheme)
{
	m_scheme = toLower(scheme);
	if (!validateScheme(m_scheme)) {
		m_is_good = false;
	}
	m_is_absolute = true;
}

void
uri::setRawAuthority(const std::string& authority)
{
	m_has_authority = true;
	parseAuthority(authority);
	updateSchemeSpecificPart();
}

void
uri::removeAuthority()
{
	m_has_authority = false;
	m_has_user_info = false;
	m_raw_user_info.erase();
	m_host.erase();
	m_port = -1;
	updateSchemeSpecificPart();
}


void
uri::setRawUserInfo(const std::string& user_info)
{
	m_raw_user_info = user_info;
	m_has_user_info = true;
	updateAuthority();
	updateSchemeSpecificPart();
}

void
uri::setHost(const std::string& host)
{
	m_host = host;
	if (!validateHost(m_host)) {
		m_is_good = false;
	}
	m_has_authority = true;
	updateAuthority();
	updateSchemeSpecificPart();
}

int
uri::guessPort() const
{
	if (m_port >= 0) {
		return m_port;
	} else {
		if (m_scheme == "http") {
			return 80;
		} else if (m_scheme == "https") {
			return 443;
		}
	}
	return -1;
}

void
uri::setPort(int port)
{
	m_port = port;
	updateAuthority();
	updateSchemeSpecificPart();
}

void
uri::setAbsoluteRawPath(const std::string& path)
{
	m_is_relative_path = false;
	if (!path.empty() && path[0] == '/') {
		m_raw_path = path.substr(1);
	} else {
		m_raw_path = path;
	}
	updateSchemeSpecificPart();
}

void
uri::setRawQuery(const std::string& query)
{
	m_has_query = true;
	m_raw_query = query;
	updateSchemeSpecificPart();
}

void
uri::removeQuery()
{
	m_has_query = false;
	m_raw_query.erase();
	updateSchemeSpecificPart();
}

uri_query
uri::getRawQueryComponents(char separator) const
{
	uri_query res;
	StringTokenizer stk(getRawQuery(), std::string(1, separator));
	while (stk.hasMoreTokens()) {
		std::string nv = stk.nextToken();
		std::string::size_type pos = nv.find('=');
		if (pos == std::string::npos) {
			res.push_back( std::pair<std::string,std::string>(nv, ""));
		} else {
			res.push_back(std::pair<std::string,std::string>(nv.substr(0, pos), nv.substr(pos+1)));
		}
	}
	return res;
}

uri_query
uri::getDecodedQueryComponents(char separator) const
{
	uri_query res = getRawQueryComponents(separator);
	uri_query::iterator iter = res.begin();
	for (; iter != res.end(); ++iter) {
		iter->first = uri::unescape(iter->first, true);
		iter->second = uri::unescape(iter->second, true);
	}
	return res;
}

void
uri::setRawFragment(const std::string& fragment)
{
	m_raw_fragment = fragment;
	m_has_fragment = true;
}

void
uri::removeFragment()
{
	m_raw_fragment.erase();
	m_has_fragment = false;
}

void
uri::makeRelative()
{
	m_is_absolute = false;
	m_scheme.erase();
	m_has_authority = false;
	m_has_user_info = false;
	m_raw_user_info.erase();
	m_host.erase();
	m_port = -1;
	updateSchemeSpecificPart();
}

std::string
uri::toString() const
{
	std::ostringstream strm;
	toStream(strm);
	return strm.str();
}

void
uri::toStream(std::ostream& strm) const
{
	if (isAbsolute()) {
		strm << getScheme() << ':';
	}
	strm << getRawSchemeSpecificPart();
	if (hasFragment()) {
		strm <<  '#' << getRawFragment();
	}
}

void
uri::parseSchemeSpecificPart(const std::string& str)
{
	std::string::size_type hash_pos = str.rfind('#');
	if (hash_pos == std::string::npos) {
		m_raw_scheme_specific_part = str;
		m_has_fragment = false;
		m_raw_fragment.erase();
	} else {
		m_raw_scheme_specific_part = str.substr(0, hash_pos);
		m_has_fragment = true;
		m_raw_fragment = str.substr(hash_pos+1);
	}
	std::string::size_type path_start_pos = 0;
	std::string::size_type question_pos = 0;
	if (m_raw_scheme_specific_part.substr(0, 2) == "//") {
		m_has_authority = true;
		std::string::size_type after_authority_pos = m_raw_scheme_specific_part.find_first_of("/?", 2);
		if (after_authority_pos == std::string::npos) {
			parseAuthority(m_raw_scheme_specific_part.substr(2));
			path_start_pos = std::string::npos;
			question_pos = std::string::npos;
		} else {
			parseAuthority(m_raw_scheme_specific_part.substr(2, after_authority_pos-2));
			if (m_raw_scheme_specific_part[after_authority_pos] == '/') {
				path_start_pos = after_authority_pos;
				question_pos = m_raw_scheme_specific_part.find('?', after_authority_pos+1);
			} else {
				path_start_pos = std::string::npos;
				question_pos = after_authority_pos;
			}
		}
	} else {
		m_has_authority = false;
		m_has_user_info = false;
		m_raw_user_info.erase();
		m_host.erase();
		m_port = -1;
		// leaving path_start_pos at 0
		question_pos = m_raw_scheme_specific_part.find('?');
	}
	if (path_start_pos == std::string::npos) {
		// ? follows the authority, or there is nothing after the authority
		m_is_relative_path = false;
		m_raw_path.erase();
	} else {
		std::string::size_type path_end_pos = (question_pos != std::string::npos)
			? question_pos : m_raw_scheme_specific_part.length();
		if (m_raw_scheme_specific_part.substr(path_start_pos, 1) == "/") {
			m_is_relative_path = false;
			m_raw_path = m_raw_scheme_specific_part.substr(path_start_pos+1, path_end_pos-path_start_pos-1);
		} else {
			m_is_relative_path = true;
			m_raw_path = m_raw_scheme_specific_part.substr(path_start_pos, path_end_pos-path_start_pos);
		}
	}
	if (question_pos == std::string::npos) {
		m_has_query = false;
		m_raw_query.erase();
	} else {
		m_has_query = true;
		m_raw_query = m_raw_scheme_specific_part.substr(question_pos+1);
	}
}

void
uri::updateSchemeSpecificPart()
{
	if (hasAuthority()) {
		m_raw_scheme_specific_part = "//"+getRawAuthority()+"/";
	} else {
		m_raw_scheme_specific_part = "/";
	}
	m_raw_scheme_specific_part += getRawPath();
	if (hasQuery()) {
		m_raw_scheme_specific_part += "?"+getRawQuery();
	}
	// fragment is not part of the scheme specific part
}

void
uri::parseAuthority(const std::string& str)
{
	m_has_authority = true;
	m_raw_authority = str;
	std::string::size_type at_pos = m_raw_authority.find('@');
	std::string::size_type host_start_pos = 0;
	if (at_pos == std::string::npos) {
		m_has_user_info = false;
		m_raw_user_info.erase();
	} else {
		m_has_user_info = true;
		m_raw_user_info = m_raw_authority.substr(0, at_pos);
		host_start_pos = at_pos+1;
	}
	std::string::size_type colon_pos = m_raw_authority.find(':', host_start_pos);
	if (colon_pos == std::string::npos) {
		m_host = m_raw_authority.substr(host_start_pos);
		m_port = -1;
	} else {
		m_host = m_raw_authority.substr(host_start_pos, colon_pos-host_start_pos);
		m_port = std::atoi(m_raw_authority.substr(colon_pos+1).c_str());
		if (m_port == -1) {
			m_is_good = false;
		}
	}
	if (!validateHost(m_host)) {
		m_is_good = false;
	}
}

void
uri::updateAuthority()
{
	if (hasUserInfo()) {
		m_raw_authority = getRawUserInfo()+"@"+getHost();
	} else {
		m_raw_authority = getHost();
	}
	if (m_port >= 0) {
		m_raw_authority += ":" + fromNumber(m_port);
	}
}

bool
uri::validateScheme(const std::string& scheme) const
{
	static const std::string allowed = ALNUM+"+-.";
	return (scheme.find_first_not_of(allowed) == std::string::npos);
}

bool
uri::validateHost(const std::string& host) const
{
	static const std::string allowed = ALNUM+".-%"; // percent is added to allow escaped unicode domain names
	return (host.find_first_not_of(allowed) == std::string::npos);
}
};